/*--------------------------------------------------------------------
  File:        graphics.cpp
  Author:      Nils Schmeisser
  ----------------------------------------------------------------------
  Description: Compatibility module to Borland's GRAPHICS module
  ----------------------------------------------------------------------
  Notes:       replace #include <graphics.h> by #include "graph.h"
               define USE_GL or USE_X depend on what you want to use,
               either GL or X. If USE_X is defined, set the GRAPHICS_DISPLAY
               to the display variable where you want to put your graphics
               window.
  Date:        22.11.1993
  ----------------------------------------------------------------------
*/ 

#define XTFUNCPROTO
#define ANSI

/*--- For recompiling set the USE_X or USE_GL ----------------*/

/* #define USE_GL */
#define USE_X

/*------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#define GRAPH_MODULE
  #include "xgraphics.h"
#undef GRAPH_MODULE

#ifdef USE_GL
  #include <gl/gl.h>
  #include <gl/device.h>
  
  static long winid;
#endif

#ifdef USE_X
  #include <X11/Xlib.h>
  #include <X11/Xutil.h>
  #include <X11/Xos.h>
  #include <X11/X.h>

  static unsigned long ColorOfs=0;
  static Colormap ColorTable;
  static XColor   ColorEntry[256];

  static Visual *VisualInfo;

  static GC XAppGraphicsContext;
  static XEvent XAppGraphicsEvent;

  Window GRAPHICS_WINDOW;
  Display *GRAPHICS_DISPLAY;
#endif

int MAXCOLORS;
static int sizex,sizey;
int COLOR;
int    BLACK;          /* dark colors */
int    BLUE;
int    GREEN;
int    CYAN;
int    RED;
int    MAGENTA;
int    BROWN;
int    LIGHTGRAY;
int    DARKGRAY;           /* light colors */
int    LIGHTBLUE;
int    LIGHTGREEN;
int    LIGHTCYAN;
int    LIGHTRED;
int    LIGHTMAGENTA;
int    YELLOW;
int    WHITE;


/*--------------------------------------------------------------------------*/

#ifdef USE_X
int ConvRGBtoPIX(int red,int green,int blue) {
  XColor q;
  int i;
  long dist,min=256*256*3+1;
  unsigned long c;

  for (i=0;i<256;i++) {
//    q.pixel=(unsigned long)i;
//    XQueryColor(GRAPHICS_DISPLAY,ColorTable,&q);
    q.red  =ColorEntry[i].red   ;
    q.green=ColorEntry[i].green ;
    q.blue =ColorEntry[i].blue  ;
    dist=( (q.red-red)*(q.red-red)
	  +(q.green-green)*(q.green-green)
	  +(q.blue-blue)*(q.blue-blue));
    if (dist<min) {
      min=dist;
      c=i;
    }
  }
  return(c);
}
#endif

extern void setrgbpalette(int colornum, int red, int green, int blue) {
  #ifdef USE_GL
    mapcolor(colornum,red,green,blue);
  #endif
  #ifdef USE_X  
    ColorEntry[colornum+ColorOfs].pixel  =(unsigned long)(colornum+ColorOfs);
    ColorEntry[colornum+ColorOfs].red    =(unsigned short)(red*256);
    ColorEntry[colornum+ColorOfs].green  =(unsigned short)(green*256);
    ColorEntry[colornum+ColorOfs].blue   =(unsigned short)(blue*256);
    ColorEntry[colornum+ColorOfs].flags  = DoRed | DoGreen | DoBlue;    
    XStoreColor(GRAPHICS_DISPLAY,ColorTable,&ColorEntry[colornum+ColorOfs]);
  #endif
}

void SetColors_to_VGA() {
  setrgbpalette(BLACK       ,  0,  0,  0);
  setrgbpalette(BLUE        ,  0,  0,127);
  setrgbpalette(GREEN       ,  0,127,  0);
  setrgbpalette(CYAN        ,  0,127,127);
  setrgbpalette(RED         ,127,  0,  0);
  setrgbpalette(MAGENTA     ,127,  0,127);
  setrgbpalette(BROWN       ,127,127,  0);
  setrgbpalette(LIGHTGRAY   ,127,127,127);
  setrgbpalette(DARKGRAY    , 63, 63, 63);
  setrgbpalette(LIGHTBLUE   ,  0,  0,255);
  setrgbpalette(LIGHTGREEN  ,  0,255,  0);
  setrgbpalette(LIGHTCYAN   ,  0,255,255);
  setrgbpalette(LIGHTRED    ,255,  0,  0);
  setrgbpalette(LIGHTMAGENTA,255,  0,255);
  setrgbpalette(YELLOW      ,255,255,  0);
  setrgbpalette(WHITE       ,255,255,255);
}

void cleardevice() {
  #ifdef USE_GL
    color(BLACK);
    clear();
  #endif
  #ifdef USE_X
    XClearWindow(GRAPHICS_DISPLAY,GRAPHICS_WINDOW);
  #endif
}

#ifdef USE_X
void XGraphicsOn() {
  GRAPHICS_DISPLAY=XOpenDisplay((char*)NULL);
}

void XGraphicsOff() {
  XCloseDisplay(GRAPHICS_DISPLAY);
}
#endif

/*--------------------------------------------------------------------------*/

static int _GRAPHICSX,_GRAPHICSY;

void setgraphicssize(int w,int h) {
  _GRAPHICSX=w;
  _GRAPHICSY=h;
}

void initgraph(int *d,int *m,char *path) {
  char driver[40];
  #ifdef USE_X
    int i;
    Window root;
    int depth,screen;
    unsigned int border_width = 0;
    int x=0,y=0; 
    XColor q;

    XGCValues GCValues;
    XSetWindowAttributes Attr;
  #endif

  switch (*d) {
   case VGA: switch (*m) {
               case VGAHI:    sizex=640;
                              sizey=480;
                              MAXCOLORS=16;
                              strcpy(driver,"Standard VGA 640*480*16");
                              break;
               case VGAHI256: sizex=640;
                              sizey=480;
                              MAXCOLORS=256;
                              strcpy(driver,"Super VGA 640*480*256");
                              break;
               case   SVGA:   sizex=800;
                              sizey=600;
                              MAXCOLORS=16;
                              strcpy(driver,"Super VGA 800*600*16");
                              break;
               case SVGA256:  sizex=800;
                              sizey=600;
                              MAXCOLORS=256;
                              strcpy(driver,"Super VGA 800*600*256");
                              break;
               case SVGATRUE: sizex=800;
                              sizey=600;
                              MAXCOLORS=16200;   /* Means 16.2 Million, but MAXINT is about 32000 */
                              strcpy(driver,"Super VGA 800*600*16.2 Mill");
                              break;
	       case VGAFREE:  sizex=_GRAPHICSX;
		              sizey=_GRAPHICSY;
                              MAXCOLORS=256;
		              sprintf(driver,"VGA %d*%d*256",sizex,sizey);
                              break;
	     }
             break;

  }
  #ifdef USE_GL
    prefposition(10,long(sizex)+10,1000-long(sizey),1000);
    winid=winopen(driver);
    if ((*m)==SVGATRUE) {
      RGBmode();
      gconfig();
    }
    SetColors_to_VGA();
  #endif
  #ifdef USE_X
    XGraphicsOn();
    screen=XDefaultScreen(GRAPHICS_DISPLAY);
    root  =XDefaultRootWindow(GRAPHICS_DISPLAY);
    depth =XDefaultDepth(GRAPHICS_DISPLAY,screen);
    VisualInfo=XDefaultVisual(GRAPHICS_DISPLAY,screen);

    MAXCOLORS=256;
    ColorTable=XDefaultColormap(GRAPHICS_DISPLAY,screen);
    for (i=0;i<256;i++) {
      q.pixel=(unsigned long)i;
      XQueryColor(GRAPHICS_DISPLAY,ColorTable,&q);
      q.red /=256;
      q.green/=256;
      q.blue/=256;
      ColorEntry[i]=q;
//      printf("%3d: %4d %4d %4d\n",i,q.red,q.green,q.blue);
    }

  BLACK       =ConvRGBtoPIX(  0,  0,  0);
  BLUE        =ConvRGBtoPIX(  0,  0,127);
  GREEN       =ConvRGBtoPIX(  0,127,  0);
  CYAN        =ConvRGBtoPIX(  0,127,127);
  RED         =ConvRGBtoPIX(127,  0,  0);
  MAGENTA     =ConvRGBtoPIX(127,  0,127);
  BROWN       =ConvRGBtoPIX(127,127,  0);
  LIGHTGRAY   =ConvRGBtoPIX(127,127,127);
  DARKGRAY    =ConvRGBtoPIX( 63, 63, 63);
  LIGHTBLUE   =ConvRGBtoPIX(  0,  0,255);
  LIGHTGREEN  =ConvRGBtoPIX(  0,255,  0);
  LIGHTCYAN   =ConvRGBtoPIX(  0,255,255);
  LIGHTRED    =ConvRGBtoPIX(255,  0,  0);
  LIGHTMAGENTA=ConvRGBtoPIX(255,  0,255);
  YELLOW      =ConvRGBtoPIX(255,255,  0);
  WHITE       =ConvRGBtoPIX(255,255,255);

    Attr.event_mask=ExposureMask | ButtonPressMask | ColormapChangeMask | LeaveWindowMask | EnterWindowMask;
    Attr.background_pixel=0;
    Attr.border_pixel    =1;
    Attr.colormap  =ColorTable;
    GRAPHICS_WINDOW=XCreateWindow(GRAPHICS_DISPLAY,root,
                                  x,y,sizex,sizey,border_width,depth,
                                  InputOutput,VisualInfo,
                                  CWBackPixel | CWEventMask | CWBorderPixel | CWColormap,&Attr);

    GCValues.function  =GXcopy;
    GCValues.plane_mask=AllPlanes;
    GCValues.foreground=1;
    GCValues.background=0;
    XAppGraphicsContext=XCreateGC(GRAPHICS_DISPLAY, root /* GRAPHICS_WINDOW */ ,
                                  GCFunction | GCPlaneMask | GCForeground | GCBackground,&GCValues); 

    XMapWindow(GRAPHICS_DISPLAY,GRAPHICS_WINDOW);
//    XInstallColormap(GRAPHICS_DISPLAY,ColorTable);
    XSelectInput(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,
                 ButtonPressMask | ButtonReleaseMask | PointerMotionMask | KeyPressMask |
                 ColormapChangeMask | ExposureMask | LeaveWindowMask | EnterWindowMask);
    XNextEvent(GRAPHICS_DISPLAY,&XAppGraphicsEvent);
  #endif
  COLOR=WHITE;  
  cleardevice();
}

void closegraph() {
  #ifdef USE_GL
    winclose(winid);
  #endif
  #ifdef USE_X
//    XUninstallColormap(GRAPHICS_DISPLAY,ColorTable);
    XFreeColormap(GRAPHICS_DISPLAY,ColorTable);
    XFreeGC(GRAPHICS_DISPLAY,XAppGraphicsContext);
    XDestroyWindow(GRAPHICS_DISPLAY,GRAPHICS_WINDOW);	
    XGraphicsOff();
  #endif
}

int getmaxcolor() {
  return(MAXCOLORS);
}

int getmaxx() {
  return(sizex);
}

int getmaxy() {
  return(sizey);
}

void setcolor(int color) {
  COLOR=color;
}

void putpixel(int x,int y,int col) {
  #ifdef USE_GL
    color(col);
    rect(x,getmaxy()-y,x+1,getmaxy()-y-1);
  #endif
  #ifdef USE_X
    XSetForeground(GRAPHICS_DISPLAY,XAppGraphicsContext,col+ColorOfs);
    XDrawPoint(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,XAppGraphicsContext,x,y);
//    XFlush(GRAPHICS_DISPLAY);
  #endif
}

void flush() { XFlush(GRAPHICS_DISPLAY); }

void line(int x1,int y1,int x2,int y2) {
  long p1[2],p2[2];

  #ifdef USE_GL
    color(COLOR);
    p1[0]=x1;
    p1[1]=getmaxy()-y1;
    p2[0]=x2;
    p2[1]=getmaxy()-y2;
    bgnline();
      v2i(p1);
      v2i(p2);
    endline();
  #endif
  #ifdef USE_X
    XSetForeground(GRAPHICS_DISPLAY,XAppGraphicsContext,COLOR+ColorOfs);
    XDrawLine(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,XAppGraphicsContext,x1,y1,x2,y2);	
    XFlush(GRAPHICS_DISPLAY);
  #endif
}

void putRGBpixel(int x,int y,int r,int g,int b) {
  #ifdef USE_GL
    RGBcolor(r,g,b);
    rect(x,getmaxy()-y,x+1,getmaxy()-y-1);
  #endif
  #ifdef USE_X
    XSetForeground(GRAPHICS_DISPLAY,XAppGraphicsContext,ConvRGBtoPIX(r,g,b));
    XDrawPoint(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,XAppGraphicsContext,x,y);
//    XFlush(GRAPHICS_DISPLAY);
  #endif
}

unsigned long imagesize(int x1,int y1,int x2,int y2) {
  unsigned long s;

  s=(unsigned long)(x2-x1+1)*(unsigned long)(y2-y1+1)+4;
  return s;
}
/*
void getimage(int x1,int y1,int x2,int y2,void *bitmap) {
  unsigned short w,h,*d;
  char *data;

  data=(char*)bitmap;
  d=(unsigned short*)bitmap;
  data++;
  data++;
  data++;
  data++;

  w=x2-x1+1;
  h=y2-y1+1;
  (*d)=w;
  d++;
  (*d)=h;

  #ifdef USE_GL
    Int16 *dat;
    long i,j;

    dat=new Int16 [w*h];
    rectread(x1,y1,x2,y2,dat);
    for (j=h-1;j>=0;j--) {
      for (i=0;i<w;i++) {
        data[((h-1)-j)*w+i]=(char)dat[j*w+i];
      }
    }
    delete dat;
  #endif
  #ifdef USE_X
    XImage *p;  
    unsigned long i,j,o;

    p=XCreateImage(GRAPHICS_DISPLAY,VisualInfo,8,ZPixmap,0,data,w,h,8,w);
    (*p)=*XGetImage(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,x1,y1,w,h,0xffffffff,ZPixmap);
    o=p->bytes_per_line-w;
    for (i=0;i<h;i++) for (j=0;j<w;j++) data[i*w+j]=p->data[i*(w+o)+j];
  #endif
}

void putimage(int x,int y,void *bitmap,int flag) {
  unsigned short w,h,*d;
  char *data;

  data=(char*)bitmap;
  d=(unsigned short*)bitmap;
  data++;
  data++;
  data++;
  data++;
  w=(*d);
  d++;
  h=(*d);

  #ifdef USE_GL
    Int16 *dat;
    long i,j;

    dat=new Int16 [w*h];
    for (j=h-1;j>=0;j--) {
      for (i=0;i<w;i++) {
        dat[j*w+i]=(Int16)data[((h-1)-j)*w+i];
      }
    }
    rectwrite(x,y,x+w-1,y+h-1,dat);
    delete dat;
  #endif
  #ifdef USE_X
    XImage *p;
    p=XCreateImage(GRAPHICS_DISPLAY,VisualInfo,8,ZPixmap,0,data,w,h,8,w);
    XPutImage(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,XAppGraphicsContext,p,0,0,x,y,w,h);
  #endif
}
*/

int XGetMouseX() {
  Window root,child;
  int rx,ry,cx,cy;
  unsigned int mask;

  XQueryPointer(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,&root,&child,&rx,&ry,&cx,&cy,&mask);
  if (child==GRAPHICS_WINDOW) 
    return cx;
  else
    return -1;
}

int XGetMouseY() {
  Window root,child;
  int rx,ry,cx,cy;
  unsigned int mask;

  XQueryPointer(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,&root,&child,&rx,&ry,&cx,&cy,&mask);
  if (child==GRAPHICS_WINDOW) 
    return cy;
  else
    return -1;
}

int XGetMouseButtons() {
  Window root,child;
  int rx,ry,cx,cy;
  unsigned int mask;

  XQueryPointer(GRAPHICS_DISPLAY,GRAPHICS_WINDOW,&root,&child,&rx,&ry,&cx,&cy,&mask);
  if (child==GRAPHICS_WINDOW) 
    return mask;
  else
    return -1;
}

