/*--------------------------------------------------------------------
  File:        graphics.h
  Author:      Nils Schmeisser
  ----------------------------------------------------------------------
  Description: Compatibility module to Borland's GRAPHICS module, include
  ----------------------------------------------------------------------
  Notes:       replace #include <graphics.h> by #include "graph.h"
               define USE_GL or USE_X depend on what you want to use,
               either GL or X, in "deflib.h".
               If USE_X is defined, set the GRAPHICS_DISPLAY to the 
               display variable where you want to put your graphics
               window.
  Date:        22.11.1993
  ----------------------------------------------------------------------
*/ 

#ifndef GRAPHICS
  #define GRAPHICS

/* Graphics drivers */

const int DETECT   =0;
const int CGA      =1;
const int MCGA     =2;
const int EGA      =3;
const int EGA64    =4;
const int EGAMONO  =5;
const int IBM8514  =6;
const int HERCMONO =7;
const int ATT400   =8;
const int VGA      =9;
const int PC3270   =10;

/* Graphics modes */

const int CGAC0     =0; /* 320*200 palette 0 */
const int CGAC1     =1; /* 320*200 palette 1 */
const int CGAC2     =2; /* 320*200 palette 2 */
const int CGAC3     =3; /* 320*200 palette 3 */
const int CGAHI     =4; /* 640*200*2 */

const int MCGAC0    =0; /* 320*200 palette 0 */
const int MCGAC1    =1; /* 320*200 palette 1 */
const int MCGAC2    =2; /* 320*200 palette 2 */
const int MCGAC3    =3; /* 320*200 palette 3 */
const int MCGAMED   =4; /* 640*200*2 */
const int MCGAHI    =5; /* 640*480*2 */

#define VGAHI       2 /* 640*480*16 */
#define VGAHI256    3 /* 640*480*256 */
#define SVGA        4 /* 800*600*16 */
#define SVGA256     5 /* 800*600*256 */
#define SVGATRUE    6 /* 800*600*16.2Mill */
#define VGAFREE    -1 /* non standard mode, requires call to setgraphicssize(w,h) */

extern int    BLACK;          /* dark colors */
extern int    BLUE;
extern int    GREEN;
extern int    CYAN;
extern int    RED;
extern int    MAGENTA;
extern int    BROWN;
extern int    LIGHTGRAY;
extern int    DARKGRAY;           /* light colors */
extern int    LIGHTBLUE;
extern int    LIGHTGREEN;
extern int    LIGHTCYAN;
extern int    LIGHTRED;
extern int    LIGHTMAGENTA;
extern int    YELLOW;
extern int    WHITE;

#ifndef GRAPH_MODULE

  #ifdef USE_X
    #include <X11/Xlib.h>
    #include <X11/Xutil.h>
    #include <X11/Xos.h>
    #include <X11/X.h>
    
    extern Display *GRAPHICS_DISPLAY;
    extern Window GRAPHICS_WINDOW;
  #endif

  extern int MAXCOLORS;

  extern void setgraphicssize(int w,int h);
  extern void initgraph(int *d,int *m,char *path);
  extern void closegraph();
  extern void cleardevice();

  extern int XGetMouseX();
  extern int XGetMouseY();
  extern int XGetMouseButtons();

  extern int getmaxx();
  extern int getmaxy();

  extern void setrgbpalette(int colornum, int red, int green, int blue);
  extern int setcolor(int color);
  extern int getmaxcolor();

  extern void putpixel(int x,int y,int col);
  extern void line(int x1,int y1,int x2,int y2);
  extern void putRGBpixel(int x,int y,int r,int g,int b);

  extern unsigned long imagesize(int x1,int y1,int x2,int y2);
  extern void getimage(int x1,int y1,int x2,int y2,void *bitmap);
  extern void putimage(int x,int y,void *bitmap,int flag);

  extern void flush();
#endif
#endif











